@tool
class_name TypewriterEffect
extends RichTextEffect
## BBCode effect which procedurally shows the text character by character, from start to end.
## Tag params:
## - speed - Speed at which text is displayed, in characters per second.
## - delay - Delay before displaying first character, in seconds.

signal typing
signal on_frame_process_start

# To use this effect:
# - Enable BBCode on a RichTextLabel.
# - Instead of instantiating this effect directly, use a `TypewriterLabel` node.
# - Use [type speed=10.0 delay=0.0]hello[/type] in text.
var bbcode: String = "type"

var _force_visible := false


func _init(force_visible_signal: Signal) -> void:
	if force_visible_signal:
		force_visible_signal.connect(_set_force_visible)


func _process_custom_fx(char_fx: CharFXTransform) -> bool:
	if not _force_visible:
		var speed: float = char_fx.env.get("speed", Game.settings.default_text_speed)
		var delay: float = char_fx.env.get("delay", 0.0)

		char_fx.visible = (char_fx.elapsed_time - delay) * speed >= char_fx.relative_index

		if not char_fx.visible:
			typing.emit()

	return true


func _set_force_visible() -> void:
	_force_visible = true
