class_name TypewriterLabel extends RichTextLabel
## RichTextLabel with a managed typewriter effect.
## Allows for signal-based monitoring of a typewriter effect's status.
## In most cases you'll want to use this rather than instantiating TypewriterEffect yourself.
## Use `display_text` to set the text for this label.

## Emitted on the first frame that all characters in the typewriter effect are visible.
signal typing_finished

## Emit this to force the typewriter effect to finish on the next update.
## Useful for advancing skippable dialogue boxes.
signal force_visible

var _finished: bool = true
var _typing: bool = false


func _init() -> void:
	bbcode_enabled = true
	var effect := TypewriterEffect.new(force_visible)
	effect.typing.connect(_on_typing)
	install_effect(effect)


## Is the typewriter effect finished?
func is_finished() -> bool:
	return _finished


## Reset the manager's progress monitor.
## This is called automatically when setting the label text with `set_text`
func reset() -> void:
	_finished = false


## Clear the text box and set a new line of text.
## The `finished` signal will be emitted when the text is done displaying.
func display_text(text: String) -> void:
	reset()
	clear()
	append_text(text)


func _process(_delta: float) -> void:
	if not _finished:
		if not _typing:
			_finished = true
			typing_finished.emit()
		_typing = false


func _on_typing() -> void:
	_typing = true
